#!/bin/bash -ex

TARGET_BUILD_KERNEL=true

function err_exit(){
	2 >& 1 echo "$@"
	exit 1
}

if [ -n "${MAKE_JOBS}" ]; then
    CPUS=${MAKE_JOBS}
else
    CPUS=$(nproc)
    #CPUS=$(($(nproc)/2))
fi

#android-build-configs/hooks/build-x15-kernels.sh
DIR_PARENT=$(dirname $0)
WORK_ROOT=$(cd ${DIR_PARENT}/../..; pwd)
#######################################################
# better to be speified in the build config file
KERNEL_BUILD_OUT="${KERNELDIR}"
KERNEL_BUILD_CONFIGS="ti_sdk_am57x_android_release_defconfig"
KERNEL_BUILD_TARGETS="zImage dtbs modules"
KERNEL_BUILD_ARCH=arm

## Check the required configs
[ -z "${KERNEL_BUILD_SRC}" ] && err_exit "KERNEL_BUILD_SRC for the kernel source directory need to be specified"
[ -z "${KERNEL_BUILD_OUT}" ] && err_exit "KERNEL_BUILD_OUT for the kernel source directory need to be specified"
[ -z "${KERNEL_BUILD_CROSS_COMPILE}" ] && err_exit "KERNEL_BUILD_CROSS_COMPILE need to be specified to build the kernel"
[ -z "${KERNEL_BUILD_CONFIGS}" ] && err_exit "KERNEL_BUILD_CONFIGS for the build configs need to be specified"
[ -z "${KERNEL_BUILD_TARGETS}" ] && err_exit "KERNEL_BUILD_TARGETS for the targets to be built need to be specified"
[ -z "${KERNEL_BUILD_ARCH}" ] && err_exit "KERNEL_BUILD_ARCH for the architecure to be used needs to be specified"


function build_kernel(){
    if [ ! -d ${KERNEL_BUILD_SRC} ]; then
        echo "The specified kernel source directory does not exist: ${KERNEL_BUILD_SRC}"
        echo "Please check and try again"
        exit 1
    fi

    cd ${KERNEL_BUILD_SRC} && make mrproper && cd -

    CMD_MAKE_KERNEL="make -j${CPUS} \
                        ARCH=${KERNEL_BUILD_ARCH} \
                        ${CLANG_OPT} \
                        ${LD_OPT} \
                        CROSS_COMPILE=${WORK_ROOT}/${KERNEL_BUILD_CROSS_COMPILE} \
                        -C ${WORK_ROOT}/${KERNEL_BUILD_SRC} \
                        O=${WORK_ROOT}/${KERNEL_BUILD_OUT} \
                        ${KERNEL_BUILD_TARGETS}"
   echo ${CMD_MAKE_KERNEL} > ${WORK_ROOT}/${KERNEL_BUILD_OUT}/build.log
   ${CMD_MAKE_KERNEL}
}

function generate_kernel_config(){
    make -j1 \
        ARCH=${KERNEL_BUILD_ARCH} \
        -C ${WORK_ROOT}/${KERNEL_BUILD_SRC} \
        O=${WORK_ROOT}/${KERNEL_BUILD_OUT} \
        ${KERNEL_BUILD_CONFIGS}
}

function build_uboot(){
    local UBOOT_DIR=${WORK_ROOT}/external/u-boot
    local UBOOT_OUT_DIR=${WORK_ROOT}/out/target/product/beagle_x15/obj/u-boot

    rm -fr ${UBOOT_OUT_DIR} && mkdir -p ${UBOOT_OUT_DIR}

    make -j$(nproc) \
        -C ${UBOOT_DIR} \
        O=${UBOOT_OUT_DIR} \
        ARCH=arm \
        CROSS_COMPILE=${WORK_ROOT}/${KERNEL_BUILD_CROSS_COMPILE} \
        am57xx_evm_defconfig

    make -j$(nproc) \
        -C ${UBOOT_DIR} \
        O=${UBOOT_OUT_DIR} \
        ARCH=arm \
        CROSS_COMPILE=${WORK_ROOT}/${KERNEL_BUILD_CROSS_COMPILE}

    cp -fv ${UBOOT_OUT_DIR}/u-boot.img ${WORK_ROOT}/device/ti/beagle_x15/bootloader/u-boot.img
    cp -fv ${UBOOT_OUT_DIR}/MLO ${WORK_ROOT}/device/ti/beagle_x15/bootloader/MLO
}

function main(){
	generate_kernel_config
	build_kernel 2>&1 | tee -a ${WORK_ROOT}/${KERNEL_BUILD_OUT}/build.log

	build_uboot 2>&1 | tee -a ${WORK_ROOT}/${UBOOT_OUT_DIR}/build.log

}

main "$@"
